#include "cmake-use-pragma-once/cmake-use-pragma-once.h"

#include "cmake-use-pragma-once/cmake-use-pragma-once-both.h"
#include "cmake-use-pragma-once/cmake-use-pragma-once-include-guards.h"
#include "cmake-use-pragma-once/cmake-use-pragma-once-neither.h"
