// Correction needed

bool function1(bool i)
{
  return true;
}

int function2(bool i)
{
  return 0;
}

char function3(bool i)
{
  return 'a';
}

void function4(bool i)
{
}

float function5(bool i)
{
  return 1.0;
}

double function6(bool i)
{
  return 0;
}

// No correction needed
bool global;

bool function7(int i)
{
  bool l;
  return true;
}

int function8(int i)
{
  return i;
}

char function9(char i)
{
  return i;
}

void function10()
{
}

float function11(float i)
{
  return i;
}

double function12(double i)
{
  return i;
}
